/*********
  Rui Santos
  Complete project details at https://RandomNerdTutorials.com/esp32-esp8266-thermostat-web-server/

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files.

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  ************************************************************************************************
  ************************************************************************************************
  ************************************************************************************************

  Temperatursteuerung mit einem Arduino ESP32 oder ESP8266 und dem DS18B20
  Temperatur ist einstellbar per Display und Knöpfen, oder per Wifi über den
  Eigenen Heimserver (Router) dazu müssen aber einige Zeilen wieder einkommentiert werden,
  dazu bitte weiterlesen um die Anleitung zu finden.

  Wahlweise kann das System aber auch Offline über den Server des Arduinos (AP == Access Point)
  nur alleine Gesteuert werden.

  Geregelt werden kann die Soll Temperatur, sowie ob das ganze per Automatik läuft oder nicht.

  Wenn der eigene Wlanrouter Zuhause genutzt werden soll bitte die Zeilennummern:

  140, 142, 143, 144, 145 sowie 151 und 152 wieder einkommentieren!
  Und in Zeile 49 und 50 Deine Router Zugangsdaten eingeben.

  Ohne das Einkomentieren läuft das System nur als reiner Access Point.
  Es geht aber auch beides gemeinsam, wenn ein Router vorhanden ist!

  Ohne Router in der Nähe geht es nur als reiner Access Point.
*********/
#define ESP32 
#ifdef ESP32
#include <WiFi.h>
#include <AsyncTCP.h>
#else
#include <ESP8266WiFi.h>
#include <ESPAsyncTCP.h>
#endif
#include <ESPAsyncWebServer.h>
#include <OneWire.h>
#include <DallasTemperature.h>

// REPLACE WITH YOUR NETWORK CREDENTIALS
const char* wifi_network_ssid = "Dein_WIFI";
const char* wifi_network_password = "Dein_Kennwort";

const char* soft_ap_ssid = "Heizung";
const char* soft_ap_password = "123456789";

//minimale Temperatur
const int MIN_TEMP = 24;
//maximale Temperatur
const int MAX_TEMP = 26;

AsyncWebServer server(80);

// Default Threshold Temperature Value, ==> Start/Soll Temperatur
String inputMessage = "18.0";
String lastTemperature;
String enableArmChecked = "checked";
String inputMessage2 = "true";

// HTML web page to handle 2 input fields (threshold_input, enable_arm_input)
const char index_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html><head>
  <title>Truma Temperatur Control</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
      <style>
      body {
      background-color: lightblue;
      }
      input {
      padding: 15px 100px;
      margin:10px 4px;
      cursor: pointer;
      text-transform: uppercase;
      position: relative;
      }
      input:hover {
      opacity:0.5;}
    </style>
  </head><body>
  <h2>Truma Digital Heiz Control</h2>
  </br></br>
  <h2>Aktuelle ist Temperatur</h2>  
  <h3><p style="color: red;">%TEMPERATURE% &deg;C</p></h3>
  <form action="/get">
    Soll Temperature <input type="number" step="0.1" name="threshold_input" value="%THRESHOLD%" required><br>
    Automatik An/Aus<input type="checkbox" name="enable_arm_input" value="true" %ENABLE_ARM_INPUT%><br><br>
    <input type="submit" value="Senden">
  </form>
</body></html>)rawliteral";

void notFound(AsyncWebServerRequest *request) {
  request->send(404, "text/plain", "Not found");
}


// Replaces placeholder with DS18B20 values
String processor(const String& var) {
  //Serial.println(var);
  if (var == "TEMPERATURE") {
    return lastTemperature;
  }
  else if (var == "THRESHOLD") {
    return inputMessage;
  }
  else if (var == "ENABLE_ARM_INPUT") {
    return enableArmChecked;
  }
  return String();
}

// Flag variable to keep track if triggers was activated or not
bool triggerActive = false;

const char* PARAM_INPUT_1 = "threshold_input";
const char* PARAM_INPUT_2 = "enable_arm_input";

// Interval between sensor readings. Learn more about ESP32 timers: https://RandomNerdTutorials.com/esp32-pir-motion-sensor-interrupts-timers/
unsigned long previousMillis = 0;
const long interval = 5000;

// GPIO where the output is connected to, ==> Pin für Relais
const int output = 2;

//die einfach Relaisplatine ist am
//digitalen Pin D14 angeschlossen
const int RELAIS2 = 14;

// GPIO where the DS18B20 is connected to, ==> Daten-Pin für 1. TemperaturSensor DS18B20
const int oneWireBus = 4;
// GPIO where the DS18B20 is connected to, ==> Daten-Pin für 2. TemperaturSensor DS18B20
const int oneWireBus2 = 32;
// Setup a oneWire instance to communicate with any OneWire devices
OneWire oneWire(oneWireBus);
OneWire oneWire2(oneWireBus2);
// Pass our oneWire reference to Dallas Temperature sensor
DallasTemperature sensors(&oneWire);
DallasTemperature sensors2(&oneWire2); 

void setup() {
  Serial.begin(115200);
  WiFi.mode(WIFI_MODE_APSTA);
  WiFi.softAP(soft_ap_ssid, soft_ap_password);

  // Zum Nutzen mit eigenem Wlan (Wifi) einkommentieren
  //  WiFi.begin(wifi_network_ssid, wifi_network_password);

  //while (WiFi.status() != WL_CONNECTED) {
  //  delay(500);
  //  Serial.println("Connectingto Wifi..");
  // }

  Serial.print("ESP32 IP as soft AP: ");
  Serial.println(WiFi.softAPIP());

  // Zum Nutzen mit eigenem Wlan (Wifi) einkommentieren
  //  Serial.print("ESP32 IP on the WIFI network: ");
  //  Serial.println(WiFi.localIP());

  pinMode(output, OUTPUT);
  digitalWrite(output, LOW);


  // Start the DS18B20 sensor
  sensors.begin();
  sensors2.begin();

  //definieren das der digitale Pin des Relais
  //als Ausgang dient
  pinMode(RELAIS2, OUTPUT);
  digitalWrite(RELAIS2, LOW);

  // Send web page to client
  server.on("/", HTTP_GET, [](AsyncWebServerRequest * request) {
    request->send_P(200, "text/html", index_html, processor);
  });

  // Receive an HTTP GET request at <ESP_IP>/get?threshold_input=<inputMessage>&enable_arm_input=<inputMessage2>
  server.on("/get", HTTP_GET, [] (AsyncWebServerRequest * request) {
    // GET threshold_input value on <ESP_IP>/get?threshold_input=<inputMessage>
    if (request->hasParam(PARAM_INPUT_1)) {
      inputMessage = request->getParam(PARAM_INPUT_1)->value();
      // GET enable_arm_input value on <ESP_IP>/get?enable_arm_input=<inputMessage2>
      if (request->hasParam(PARAM_INPUT_2)) {
        inputMessage2 = request->getParam(PARAM_INPUT_2)->value();
        enableArmChecked = "checked";
      }
      else {
        inputMessage2 = "false";
        enableArmChecked = "";
      }
    }
    Serial.println(inputMessage);
    Serial.println(inputMessage2);

    request->send(200, "text/html", "HTTP GET request sent to your ESP.<br><a href=\"/\">Return to Home Page</a>");
  });
  server.onNotFound(notFound);
  server.begin();
}

void loop() {
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;
    sensors.requestTemperatures();
    // Temperature in Celsius degrees
    float temperature = sensors.getTempCByIndex(0);
    Serial.print("Temperatur des Sensors vom Raum ist: ");
    Serial.print(temperature);
    Serial.println(" C");

    // Temperature in Fahrenheit degrees
    /*float temperature = sensors.getTempFByIndex(0);
      Serial.print(temperature);
      Serial.println(" *F");*/

    lastTemperature = String(temperature);

    // Check if temperature is above threshold and if it needs to trigger output
    if (temperature > inputMessage.toFloat() && inputMessage2 == "true" && !triggerActive) {
      String message = String("Temperature above threshold. Current temperature: ") +
                       String(temperature) + String("C");
      Serial.println(message);
      triggerActive = true;
      digitalWrite(output, HIGH);
    }
    
    // Check if temperature is below threshold and if it needs to trigger output
    else if ((temperature < inputMessage.toFloat()) && inputMessage2 == "true" && triggerActive) {
      String message = String("Temperature below threshold. Current temperature: ") +
                       String(temperature) + String(" C");
      Serial.println(message);
      triggerActive = false;
      digitalWrite(output, LOW);
    }
  }

 //Anfordern der Temperaturwerte
  sensors2.requestTemperatures();
  //Auslesen der Temperatur des ersten Sensors
  //da die Sensoren über eigene Adressen verfügen, können wir mehrere
  //Sensoren des Typs DS18B20 über eine Schnittstelle anschließen
  float tempC = sensors2.getTempCByIndex(0);

  //Wenn erfolgreich eine Temperatur gelesen wurde, dann...
  if (tempC != DEVICE_DISCONNECTED_C) {
    //Ausgeben der Temperatur auf der seriellen Schnittstelle
    Serial.print("Temperatur des Sensors der Heizung ist: ");
    Serial.print(tempC);
    Serial.println(" C");
  } else {
    //andernfalls soll eine Fehlermeldung ausgegeben werden
    Serial.println("Fehler: Es konnten keine Daten ermittelt werden!");
    //verlassen der Loop Funktion
    return;
  }

  //Wenn die gelesene Temperatur kleiner / gleich als der Wert der Konstante MIN_TEMP ist,
  //dann soll das Relais aktiviert werden.
  if (tempC <= MIN_TEMP) {
    digitalWrite(RELAIS2, HIGH);
  } else if (tempC >= MAX_TEMP) {
    //Ist der gelesene Wert jedoch größer / gleich als der Wert der Konstante MAX_TEMP so
    //soll das Relais deaktiviert werden.
    digitalWrite(RELAIS2, LOW); 
  }
}
